package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

/**
 * MIDI port registered by {@link JackClient}.
 * 
 * @author repa
 * 
 */
public class JackLocalMidiPort extends JackLocalPort {

	static {
		JackBridge.initializeJackBridge();
	}

	private JackMidiPortBuffer jackMidiPortBuffer;

	JackLocalMidiPort(long portHandle, JackClient client) {
		super(portHandle, client);
	}

	/**
	 * Returns instance of {@link JackMidiPortBuffer} for this port.
	 * 
	 * <p>
	 * The returned instance is <b>only valid during current process cycle</b>,
	 * and may not be cached (even if bufferSize stays the same).
	 * 
	 * <p>
	 * For output buffer, you still need to call
	 * {@link JackMidiPortBuffer#clearBuffer()} before writing to the buffer.
	 * 
	 * @param bufferSize
	 *            current buffer size, as passed to
	 *            {@link JackClient#process(int)} callback
	 * @return valid {@link JackMidiPortBuffer} object
	 * @throws JackException
	 *             if the buffer can't be initialized
	 */
	public JackMidiPortBuffer initializeMidiBuffer(int bufferSize) throws JackException {
		long bufferPointer = getBufferPointer(bufferSize);
		if (jackMidiPortBuffer != null) {
			jackMidiPortBuffer.setBufferPointer(bufferPointer);
		} else {
			jackMidiPortBuffer = new JackMidiPortBuffer(bufferPointer);
		}

		return jackMidiPortBuffer;
	}
}
