package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

import java.util.EnumSet;

/**
 * Jack port.
 * 
 * @author repa
 * 
 */
public class JackPort {

	static {
		JackBridge.initializeJackBridge();
	}

	private volatile long portHandle = 0;

	JackPort(long portHandle) {
		this.portHandle = portHandle;
	}

	/**
	 * Returns the short name (not including 'client_name:' prefix) of this
	 * port.
	 */
	public native String getPortShortName() throws JackException;

	/**
	 * Returns the full name (including 'client_name:' prefix) of this port.
	 */
	public native String getPortName() throws JackException;

	/**
	 * Returns port type.
	 */
	public native String getPortType() throws JackException;

	/**
	 * Returns true if this is an input port.
	 */
	public boolean isInput() throws JackException {
		return (getPortFlags() & JackPortFlag.IS_INPUT.intValue()) != 0;
	}

	/**
	 * Returns true if this is an output port.
	 */
	public boolean isOutput() throws JackException {
		return (getPortFlags() & JackPortFlag.IS_OUTPUT.intValue()) != 0;
	}

	/**
	 * Returns true if this port is physical.
	 */
	public boolean isPhysical() throws JackException {
		return (getPortFlags() & JackPortFlag.IS_PHYSICAL.intValue()) != 0;
	}

	/**
	 * Returns true if this port can be monitored.
	 */
	public boolean isMonitorable() throws JackException {
		return (getPortFlags() & JackPortFlag.CAN_MONITOR.intValue()) != 0;
	}

	/**
	 * Returns true if this port is terminal.
	 */
	public boolean isTerminal() throws JackException {
		return (getPortFlags() & JackPortFlag.IS_TERMINAL.intValue()) != 0;
	}

	/**
	 * Returns the pointer to C <code>jack_port_t</code> structure,
	 * represented as <code>long</code>.
	 */
	public long getPortHandle() {
		return portHandle;
	}

	/**
	 * Returns current port flags, as reported by Jack API.
	 */
	public EnumSet<JackPortFlag> getPortFlagSet() throws JackException {
		return JackPortFlag.decode(getPortFlags());
	}

	/**
	 * Returns current port flags, packed as int.
	 */
	public native int getPortFlags() throws JackException;

	/**
	 * Returns port latency.
	 */
	public native int getLatency() throws JackException;

	/**
	 * Sets port latency.
	 */
	public native void setLatency(int latency) throws JackException;

	/**
	 * If the port can be monitored, sets monitoring on or off.
	 */
	public native void requestMonitor(boolean on) throws JackException;

	/**
	 * If the port can be monitored, this method, turns on input monitoring if
	 * it was off, and turns it off if only one request has been made to turn it
	 * on; otherwise, it does nothing.
	 */
	public native void ensureMonitor(boolean on) throws JackException;

	/**
	 * Returns true if input monitoring was requested for this port.
	 */
	public native boolean monitoringInput() throws JackException;
}
