package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

import java.util.EnumSet;
import java.util.Set;

/**
 * Java version of Jack enum <code>JackStatus</code>.
 * 
 * @author repa
 * 
 */
public enum JackStatus {
	JackFailure {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackFailure();
		}
	},
	JackInvalidOption {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackInvalidOption();
		}
	},
	JackNameNotUnique {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackNameNotUnique();
		}
	},
	JackServerStarted {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackServerStarted();
		}
	},
	JackServerFailed {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackServerFailed();
		}
	},
	JackServerError {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackServerError();
		}
	},
	JackNoSuchClient {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackNoSuchClient();
		}
	},
	JackLoadFailure {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackLoadFailure();
		}
	},
	JackInitFailure {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackInitFailure();
		}
	},
	JackShmFailure {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackShmFailure();
		}
	},
	JackVersionError {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackVersionError();
		}
	},
	;

	private Integer cachedIntValue = null;

	protected abstract int nativeIntValue();

	/**
	 * Returns integer value of native constant corresponding to this enum
	 * value.
	 */
	public int intValue() {
		if (cachedIntValue == null) {
			cachedIntValue = nativeIntValue();
		}
		return cachedIntValue;
	}

	/**
	 * Encodes set of enum constants as <code>int</code>, by ORing their
	 * values together.
	 */
	public static int encode(Set<JackStatus> flags) {
		int result = 0;
		for (JackStatus flag : flags) {
			result |= flag.intValue();
		}
		return result;
	}

	/**
	 * Decodes <code>int</code> value as set of enum values.
	 * 
	 * <p>
	 * The <code>value</code> must be a result of ORing values of native enum
	 * constants.
	 * 
	 * @throws IllegalArgumentException
	 *             if the value contains unrecognized bits
	 */
	public static EnumSet<JackStatus> decode(final int value) {
		EnumSet<JackStatus> result = EnumSet.noneOf(JackStatus.class);
		int bits = value;

		for (JackStatus flag : JackStatus.values()) {
			if ((bits & flag.intValue()) != 0) {
				result.add(flag);
				bits ^= flag.intValue();
			}
		}

		if (bits != 0) {
			throw new IllegalArgumentException("Can't decode JackStatus from value " + value); //$NON-NLS-1$
		}

		return result;
	}
}
