package net.sf.jack4j;

/*
Copyright (C) 2008 Ondrej Par

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/


/**
 * Java version of Jack enum <code>jack_transport_state_t</code>.
 * 
 * @author repa
 * 
 */
public enum JackTransportState {
	STOPPED {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackTransportStopped();
		}
	},
	ROLLING {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackTransportRolling();
		}
	},
	LOOPING {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackTransportLooping();
		}
	},
	STARTING {

		@Override
		protected int nativeIntValue() {
			return JackConstants.JackTransportStarting();
		}
	},
	;

	private Integer cachedIntValue = null;

	protected abstract int nativeIntValue();

	/**
	 * Returns integer value of native constant corresponding to this enum
	 * value.
	 */
	public int intValue() {
		if (cachedIntValue == null) {
			cachedIntValue = nativeIntValue();
		}
		return cachedIntValue;
	}

	/**
	 * Returns the enum constant with given integer value.
	 */
	public static JackTransportState byIntValue(int intValue) {
		for (JackTransportState state : values()) {
			if (state.intValue() == intValue) {
				return state;
			}
		}
		throw new IllegalArgumentException();
	}
}
